from django.conf.urls import url, include
from django.contrib.auth import views as auth_views

from rest_framework import routers

from . import views


router = routers.DefaultRouter()
router.register(r'servicegroups', views.ServiceGroupViewSet)
router.register(r'services', views.ServiceViewSet)
router.register(r'incidents', views.IncidentViewSet)
router.register(r'incidentupdates', views.IncidentUpdateViewSet)
router.register(r'maintenances', views.MaintenanceViewSet)

app_name = "statusboard"

service_urls = [
    url('^create/$',
        views.ServiceCreate.as_view(),
        name="create"),
    url('^(?P<pk>[0-9]+)/edit/$',
        views.ServiceUpdate.as_view(),
        name="edit"),
    url('^(?P<pk>[0-9]+)/delete/$',
        views.ServiceDelete.as_view(),
        name="delete"),
]

servicegroup_urls = [
    url('^create/$',
        views.ServiceGroupCreate.as_view(),
        name="create"),
    url('^(?P<pk>[0-9]+)/edit/$',
        views.ServiceGroupUpdate.as_view(),
        name="edit"),
    url('^(?P<pk>[0-9]+)/delete/$',
        views.ServiceGroupDelete.as_view(),
        name="delete"),
]

maintenance_urls = [
    url('^create/$',
        views.MaintenanceCreate.as_view(),
        name="create"),
    url('^(?P<pk>[0-9]+)/edit/$',
        views.MaintenanceUpdate.as_view(),
        name="edit"),
    url('^(?P<pk>[0-9]+)/delete/$',
        views.MaintenanceDelete.as_view(),
        name="delete"),
]

incident_urls = [
    url('^archive/$',
        views.IncidentMonthArchiveView.as_view(),
        name="archive-index"),
    url('^archive/(?P<year>[0-9]{4})/(?P<month>[0-9]{1,2})/$',
        views.IncidentMonthArchiveView.as_view(),
        name="archive-month"),
    url('^create/$',
        views.incident_create,
        name="create"),
    url('^(?P<pk>[0-9]+)/edit/$',
        views.incident_edit,
        name="edit"),
    url('^(?P<pk>[0-9]+)/delete/$',
        views.IncidentDeleteView.as_view(),
        name="delete"),
]

urlpatterns = [
    url(r'^$', views.index, name="index"),
    url(r'^services/', include(service_urls, namespace="service")),
    url(r'^servicegroups/', include(servicegroup_urls, namespace="servicegroup")),
    url(r'^incidents/', include(incident_urls, namespace="incident")),
    url(r'^maintenances/', include(maintenance_urls, namespace="maintenance")),
    url(r'^api/(?P<version>v0\.1)/', include(router.urls, namespace="api")),
    url(r'^login/$', auth_views.login, {
        "template_name": "statusboard/login.html",
    }, name="login"),
    url(r'^logout/$', auth_views.logout, {
        "template_name": "statusboard/login.html",
    }, name="logout"),
]



from django.contrib import admin

from .models import Service, ServiceGroup, Incident, IncidentUpdate, Maintenance


class IncidentUpdateInline(admin.TabularInline):
    model = IncidentUpdate
    ordering = ["created"]


class IncidentAdmin(admin.ModelAdmin):
    inlines = [IncidentUpdateInline]


admin.site.register(Service)
admin.site.register(ServiceGroup)
admin.site.register(Incident, IncidentAdmin)
admin.site.register(Maintenance)



from django import template

from statusboard.models import SERVICE_STATUSES


register = template.Library()


def service_status_text(value, arg):
    return dict(SERVICE_STATUSES).get(int(value), arg)


@register.filter
def service_status_summary_text(value, arg):
    return {
        "0": "All systems are operational",
        "1": "Some systems suffer of performance issues",
        "2": "Some systems suffer of minor outage",
        "3": "Some systems suffer of major outage",
    }.get(str(value), arg)


@register.filter
def service_status_class(value, arg):
    return {
        "0": "success",
        "1": "info",
        "2": "warning",
        "3": "danger",
    }.get(str(value), arg)


@register.filter
def service_status_glyphicon(value, arg):
    return {
        "0": "glyphicon-ok-sign",
        "1": "glyphicon-info-sign",
        "2": "glyphicon-exclamation-sign",
        "3": "glyphicon-exclamation-sign",
    }.get(str(value), arg)


@register.filter
def incident_status_class(value, arg):
    return {
        "0": "danger",
        "1": "warning",
        "2": "info",
        "3": "success",
    }.get(str(value), arg)



from rest_framework import serializers

from .models import Service, ServiceGroup, Incident, IncidentUpdate, Maintenance


class ServiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Service
        fields = ('id', 'name', 'description', 'href', 'status', 'groups',
                  'created', 'modified')


class ServiceGroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = ServiceGroup
        fields = ('id', 'name', 'services', 'created', 'modified')


class IncidentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Incident
        fields = ('id', 'name', 'service', 'occurred', 'created', 'modified',
                  'updates')


class IncidentUpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = IncidentUpdate
        fields = ('id', 'status', 'description', 'created', 'modified')


class MaintenanceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Maintenance
        fields = ('id', 'name', 'description', 'created', 'modified',
                  'scheduled')



from django.db import models
from django.utils import timezone

from model_utils.models import TimeStampedModel


SERVICE_STATUSES = (
    (0, 'Operational'),
    (1, 'Performance issues'),
    (2, 'Partial outage'),
    (3, 'Major outage'),
)


class ServiceManager(models.Manager):
    def worst_status(self):
        s = self.get_queryset().aggregate(models.Max('status'))["status__max"]
        return s


class Service(TimeStampedModel):
    name = models.CharField(max_length=255, unique=True)
    description = models.TextField()
    href = models.URLField(blank=True)
    status = models.IntegerField(choices=SERVICE_STATUSES)
    groups = models.ManyToManyField('ServiceGroup',
                                    related_name='services',
                                    related_query_name='service')
    objects = ServiceManager()

    def __str__(self):
        return self.name


class ServiceGroup(TimeStampedModel):
    name = models.CharField(max_length=255, unique=True)

    def worst_service(self):
        return self.services.all().latest('status')

    def __str__(self):
        return self.name


INCIDENT_STATUSES = (
    (0, "Investigating"),
    (1, "Identified"),
    (2, "Watching"),
    (3, "Fixed"),
)


class IncidentManager(models.Manager):
    def occurred_in_last_n_days(self, days=7):
        threshold = timezone.now() - timezone.timedelta(days=days)
        qs = self.get_queryset().filter(occurred__date__gte=threshold.date())
        return qs


class Incident(TimeStampedModel):
    name = models.CharField(max_length=255)
    service = models.ForeignKey('Service', blank=True, null=True)
    occurred = models.DateTimeField(default=timezone.now)
    objects = IncidentManager()

    def worst_status(self):
        return self.updates.aggregate(worst=models.Max('status'))['worst']

    def updates_by_ctime(self):
        return self.updates.order_by('-created')

    def __str__(self):
        return self.name


class IncidentUpdate(TimeStampedModel):
    incident = models.ForeignKey(Incident, related_name='updates',
                                 related_query_name='update')
    status = models.IntegerField(choices=INCIDENT_STATUSES)
    description = models.TextField()

    def __str__(self):
        return "Update {} {}".format(self.incident.name, self.modified)


class Maintenance(TimeStampedModel):
    scheduled = models.DateTimeField()
    name = models.CharField(max_length=255)
    description = models.TextField()

    def __str__(self):
        return self.name
